<?php
/*

REST API

This API allows a remote POST or GET request to login a user authorized to use the phone/manual order form
and display the order form, it also allows assigning a customer and populating the order with products. 

This API is typically used when integrating a 3rd party CRM with the site's order form.

The API then always responds back with either an error (WP_Error) or a URL for the order page that can be 
loaded into an iFrame, or used for a redirect, or used to load into a popup window.

USAGE: 

Send a GET request to the endpoint with parameters in a popup window or iFrame.

See the function get_endpoint_args() below for possible parameters - only user and pass are required. 

	Example: http://localhost.example/wp-json/wc-ignitewoo-order-form/v3/populate-order-form?user=salesperson&pass=test123&product_ids=101,123,2345&product_skus=adsf,2345234&customer=test@test.com


*/

class IGN_OPC_API {

	/**
	*
	* @var string
	*/
	protected $namespace = 'wc-ignitewoo-order-form/v3';
	
	private $plugin_opts = false;
	
	private $user_id = false; // user ID of the person accessing this endpoint, must be authorized to use the ordering page
	
	private $user_login = false; // user login of the person accessing this endpoint, must be authorized to use the ordering page

	public function __construct() { 

		// We do this so we can send back non-JSON responses
		add_filter( 'rest_jsonp_enabled', array( &$this, 'enable_jsonp' ), PHP_INT_MAX );
		
		add_action('rest_api_init', function(){
			$this->register_routes();
		} );
		
		add_filter( 'wp_is_application_passwords_available', '__return_true', 999999 );
	}

	public function enable_jsonp() { 
		return true; 
	}
function prefix_get_comment_schema() {
    $schema = array(
        // This tells the spec of JSON Schema we are using which is draft 4.
        '$schema'              => 'http://json-schema.org/draft-04/schema#',
        // The title property marks the identity of the resource.
        'title'                => 'comment',
        'type'                 => 'object',
        // In JSON Schema you can specify object properties in the properties attribute.
        'properties'           => array(
            'id' => array(
                'description'  => esc_html__( 'Unique identifier for the object.', 'manual_phone_orders' ),
                'type'         => 'integer',
                'context'      => array( 'view', 'edit', 'embed' ),
                'readonly'     => true,
            ),
            'author' => array(
                'description'  => esc_html__( 'The id of the user object, if author was a user.', 'manual_phone_orders' ),
                'type'         => 'integer',
            ),
            'content' => array(
                'description'  => esc_html__( 'The content for the object.', 'manual_phone_orders' ),
                'type'         => 'string',
            ),
        ),
    );

    return $schema;
}
	public function register_routes() {

		register_rest_route(
			$this->namespace,
			'/populate-order-form',
			array(
				array(
					'methods'             => array( 'POST', 'GET' ),
					'callback'            => array( $this, 'render_form' ),
					'permission_callback' => array( $this, 'get_items_permissions_check' ),
					'args'                => $this->get_endpoint_args(),
				),
				//'schema' => array( $this, 'get_item_schema' ),
			)
		);

	}

	public function get_endpoint_args() { 

		$args = array(
			 /***** REQUIRED ****/
			'user' => array(
				'description' => __( 'User name or user ID for order creation access.', 'manual_phone_orders' ),
				'type'        => 'string',
				'validate_callback' => array( &$this, 'validate_arg' ),
				'sanitize_callback' => array( &$this, 'sanitize_arg' ),
				'required' => true,
			),
			 /***** REQUIRED ****/
			'pass' => array(
				'description' => __( 'Password for user for order creation access.', 'manual_phone_orders' ),
				'type'        => 'string',
				'validate_callback' => array( &$this, 'validate_arg' ),
				'sanitize_callback' => array( &$this, 'sanitize_arg' ),
				'required' => true,
			),
			// OPTIONAL
			// WHEN CUSTOMER IS PROVIDED THEN THE CUSTOMER MUST HAVE A WP USER ACCOUNT
			'customer' => array(
				'description' => __( 'Customer ID or email address for the order being created.', 'manual_phone_orders' ),
				'type'        => 'string', // ONLY PROVIDE ID OR EMAIL, NOT BOTH
				'validate_callback' => array( &$this, 'validate_arg' ),
				'sanitize_callback' => array( &$this, 'sanitize_arg' ),
				'required' => false,
			),
			// OPTIONAL
			// NUMERIC PRODUCT ID NUMBERS
			// SIMPLE AND VARIATION PRODUCTS ONLY!
			'product_ids' => array(
				'description' => __( 'Product IDs to add to the order.', 'manual_phone_orders' ),
				'type'        => 'array_of_integers', // ARRAY OF PRODUCT IDS
				'validate_callback' => array( &$this, 'validate_arg' ),
				'sanitize_callback' => array( &$this, 'sanitize_arg' ),
				'required' => false,
			),
			// OPTIONAL
			// PRODUCT SKUs
			// SIMPLE AND VARIATION PRODUCTS ONLY!
			'product_skus' => array(
				'description' => __( 'Product SKUs to add to the order.', 'manual_phone_orders' ),
				'type'        => 'array_of_strings', // ARRAY OF PRODUCT SKUS
				'validate_callback' => array( &$this, 'validate_arg' ),
				'sanitize_callback' => array( &$this, 'sanitize_arg' ),
				'required' => false,
			),
			/*
			'redirect' => array(
				'description' => __( 'When sent the API will issue an HTTP redirect to the order page. When not sent the API will return the URL to the page.', 'manual_phone_orders' ),
				'type'        => 'string', // "true" or "false" 
				'validate_callback' => array( &$this, 'validate_arg' ),
				'sanitize_callback' => array( &$this, 'sanitize_arg' ),
				'required' => true,
			),
			*/
		);
		
		return $args;
	}
	
	function validate_arg( $value, $request, $param ) {
	
		// When this function runs the REST request has been routed, 
		// so send this header so that the browser does not expect JSON formatted data 
		header( 'Content-type: text/plain' );
		
		$attributes = $request->get_attributes();

		if ( isset( $attributes['args'][ $param ] ) ) {

			$argument = $attributes['args'][ $param ];
			
			// Check to make sure our argument is a string.
			if ( 'string' === $argument['type'] && ! is_string( $value ) ) {
				
				die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( esc_html__( '%1$s is not of type %2$s', 'manual_phone_orders' ), $param, 'string' ) ) ) );
				
			} else if ( 'integer' === $argument['type'] && absint( $value ) <= 0 ) {
			
				die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( esc_html__( '%1$s is not of type %2$s', 'manual_phone_orders' ), $param, 'integer' ) ) ) );
				
			} else if ( 'array_of_integers' === $argument['type'] ) {
			
				$data = explode( ',', $_REQUEST['product_ids'] );

				foreach( $data as $int ) { 
					if ( absint( $int ) <= 0 ) { 
						die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( esc_html__( '%1$s (' . $int . ') is not of type %2$s', 'manual_phone_orders' ), $param, 'integer' ) ) ) );
					}
				}
			}
			
		} else {
			
			// This code won't execute because we have specified this argument as required.
			// If we reused this validation callback and did not have required args then this would fire.
			die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( esc_html__( '%s was not registered as a request argument.', 'manual_phone_orders' ), $param ) ) ) );
		}
		
		// If we got this far then the data is valid.
		return true;
	}
	
	public function sanitize_arg( $param ) { 
		if ( isset( $attributes['args'][ $param ] ) ) {

			$argument = $attributes['args'][ $param ];

			if ( 'string' === $argument['type'] ) {
				return sanitize_text_field( $value );
			} else if ( 'integer' === $argument['type'] ) {
				return absint( $value );
			} else if ( 'array' === $argument['type'] ) {
				return array_map( 'absint', $param );
			} else { 
				die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( esc_html__( '%s was not registered as a request argument.', 'manual_phone_orders' ), $param ) ) ) );
			}
		}
	}

	
	public function get_items_permissions_check( $request = false ) { 
		global $ign_opc; 
		
		// check if user is allowed

		if ( empty( $_REQUEST['user'] ) || empty( $_REQUEST['pass'] ) ) { 
			die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( 'Invalid authorized user data', 'manual_phone_orders' ) ) ) );
		}

		$this->plugin_opts = get_option( 'woocommerce_ignitewoo_manual_order_settings' );
		
		if ( empty( $this->plugin_opts  ) ) { 
			die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( 'Plugin settings not saved yet', 'manual_phone_orders' ) ) ) );
		}

		if ( 'yes' !== $this->plugin_opts ['enable'] ) {
			die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( 'Plugin is not enabled via settings', 'manual_phone_orders' ) ) ) );
		}

		$user = $_REQUEST['user'];
		$pass = $_REQUEST['pass'];

		if ( absint( $_REQUEST['user'] ) > 0 ) { 
			$user_data = get_user_by( 'id', absint( $_REQUEST['user'] ) );
		} else { 
			$user_data = get_userdatabylogin( $user );
		}

		if ( empty( $user_data ) ) { 
			die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( 'Invalid authorized user data', 'manual_phone_orders' ) ) ) );
		}

		$user = $user_data->data->user_login;
		
		$user = wp_authenticate( $user, $pass );

		if ( is_wp_error( $user ) ) { 
			die( json_encode( array( 'result' => 'REST API ERROR', 'message' => sprintf( 'Invalid authorized user data', 'manual_phone_orders' ) ) ) );
		}

		if ( !empty( $user->caps['administrator'] ) || !empty( $user->caps['shop_manager'] ) ) { 
			$this->user_id = $user_data->data->ID;
		
			$this->user_login = $user_data->data->user_login;
		
			return true;
		}
		
		$has_access = false; 
		
		if ( !empty( $opts['roles'] ) ) { 
			foreach( $opts['roles'] as $role ) {
				if ( !empty( $user->caps[ $role ] ) ) { 
					$has_access = true;
				}
			}
		}
		
		if ( !empty( $opts['users'] ) ) { 
			foreach( $opts['users'] as $user_id ) {
				if ( $user->ID == $user_id  ) {
					$has_access = true;
				}
			}
		}

		$this->user_id = $user_data->data->ID;
		
		$this->user_login = $user_data->data->user_login;
		
		return $has_access;
	}

	public function render_form() {
		global $wpdb, $user_ID;

		$args = wp_parse_args( $_REQUEST );
		
		if ( !empty( $args['product_ids'] ) ) { 
			$args['product_ids'] = explode( ',', $args['product_ids'] );
			$args['product_ids'] = array_map( 'absint', $args['product_ids'] );
			// Ensure ID numbers are valid
			foreach( $args['product_ids'] as $k => $v ) { 
				if ( $v <= 0 ) { 
					unset( $args['product_ids'][$k] );
				}
			}
		}
		
		if ( !empty( $args['product_skus'] ) ) { 
			$args['product_skus'] = explode( ',', $args['product_skus'] );
			$args['product_skus'] = array_map( 'trim', $args['product_skus'] );
		}
		
		$customer_id = false;

		if ( !empty( $args['customer'] ) && is_email( $args['customer'] ) ) { 
			
			$customer_user = get_user_by( 'email', $args['customer'] );
		
			if ( !empty( $customer_user ) && !is_wp_error( $customer_user ) ) { 
				$customer_id = $customer_user->data->ID;
			}
			
		} else if ( !empty( $args['customer'] ) && absint( $args['customer'] ) > 0 ) { 
			$customer_id = absint( $args['customer'] );
		}

		// Login the user
		wp_set_current_user( $this->user_id, $this->user_login );
		
		wp_set_auth_cookie( $this->user_id );

		// Load cart if this function exists, otherwise the cart should in theory already be loaded in older versions of WC:
		if ( function_exists( 'wc_load_cart' ) ) { 
			wc_load_cart();
		}
		
		// Populate the cart 
		WC()->cart->empty_cart();
		
		if ( !empty( $args['product_skus'] ) ) { 
			foreach( $args['product_skus'] as $sku ) { 	
				$sql = $wpdb->prepare( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key='_sku' AND meta_value='%s' LIMIT 1", $sku );

				$id = $wpdb->get_var( $sql );

				if ( !empty( $id ) ) { 
					$args['product_ids'][] = absint( $id );
				}
			}
		}

		if ( !class_exists( 'IGN_OPC_Cart' ) ) { 
			require_once( dirname( __FILE__ ) . '/class-ign-opc-core.php' );
			require_once( dirname( __FILE__ ) . '/class-ign-cart.php' );
		}
		
		ob_start();
		IGN_OPC_Cart::ign_one_page_searchable_add( $args['product_ids'], TRUE );
		$x = ob_get_clean();
		
		// Get the ordering page URL 
		$url = get_permalink( $this->plugin_opts['manual_order_page'] );
		
		if ( empty( $url ) ) { 
			die( 'REST API ERROR: ' .  sprintf( esc_html__( 'The URL for the ordering page is not valid. Check the Manual/Phone Order plugin settings.', 'manual_phone_orders' ), $param ) );
		}
		
		if ( !empty( $customer_id ) ) { 
			$url = add_query_arg( array( 'one_page_customer_user' => $customer_id ), $url );
		}

		//if ( 'true' == $args['redirect'] ) { 
		//	wp_redirect( $url ); 
		//} else if ( 'false' == $args['redirect'] ) {
			die( json_encode( array( 'result' => 'success', 'message' => $url ) ) );
		//}
		
		die;
	}
}

new IGN_OPC_API();
