<?php
/*
Plugin Name: WooCommerce Phone Orders & Manual Orders
Plugin URI: https://ignitewoo.com/
Description: Lets you manually create a orders, calculate shipping, take payment, etc.
Version: 3.4.1
Author: IgniteWoo.com
Author URI: https://ignitewoo.com/
Text Domain: manual_phone_orders
Domain Path: languages/
WC requires at least: 3.0
WC tested up to: 8.1
X-IGN-ID: 22611
X-IGN-Signature: 2a0aca041ccc02ea8b50ffcac00b9583
*/

/*
	Copyright (c) 2014 - 2022 - IgniteWoo.com - All Rights Reserved
*/

/*
	Add button to the admin order detail area to open the order in the manual order interface
	
	Add setting to hide items that are out of stock
	
	Add setting to hide variable parent products from search results
	
	Add setting to cause arbitrary product's weight to be required
*/

class IGN_Manual_Phone_Orders {

	var $opc = false;

	var $style = '';

	var $products = array();

	function __construct() {

		$this->plugin_dir = dirname( __FILE__ );

		register_activation_hook( __FILE__, array( &$this, 'maybe_schedule_cron_hook' ) );
		
		add_action( 'init', array( &$this, 'init' ), 4 );

		add_action( 'plugins_loaded', array( &$this, 'plugins_loaded' ), 8 );

		// Add the gateway to the list of available payment gateway
		add_filter( 'woocommerce_payment_gateways', array( &$this, 'add_dummy_gateway' ) );
		
		// This is a bit hacky, it tries to set the customer as the active global user so that payment gateways 
		// can hopefully obtained saved cards for the customer properly. Somewhere during the call to 
		// woocommerce_checkout_payment() the user context is switched to the logged in user, which 
		// gives us the wrong user context, so use this hacky way of setting it to customer. 
		// This filter runs after WC handles getting the enabled gateways.
		add_filter( 'woocommerce_available_payment_gateways', array( &$this, 'force_customer_user' ) );

		// Add the shipping module
		add_filter( 'woocommerce_shipping_methods', array( &$this, 'add_shipping_method' ) );

		// Add this hook to replace the shortcode with an error message when the user does not have access,
		// if the user has access the core class will load and replace the shortcode.
		add_shortcode( 'woocommerce_manual_phone_order', array( &$this, 'access_is_denied' ) );

		add_action( 'init', array( &$this, 'load_plugin_textdomain' ) );
		
		add_action( 'woocommerce_checkout_billing', array( &$this, 'checkout_billing' ), -1 );

		add_action( 'woocommerce_checkout_shipping', array( &$this, 'checkout_shipping' ), -1 );
		
		// Not implemented yet
		//add_filter( 'woocommerce_product_backorders_allowed', array( &$this, 'backorders_allowed' ), 9999, 2 );
		//add_filter( 'woocommerce_is_sold_individually', array( &$this, 'sold_individually' ), 9999, 2 );

		// Not implemented yet
		add_action( 'init', array( &$this, 'load_request_vars' ), -1 );
		add_filter( 'woocommerce_is_purchasable', array( &$this, 'is_purchasable' ), PHP_INT_MAX , 2 );
		
		// Cron
		add_action( 'remove_arbitrary_products_hook', 'IGN_Arbitrary_Products::remove_arbitrary_products' );
		
		add_action( 'before_woocommerce_init', array( &$this, 'declare_hpos_compatible' ) );
	}

	public function declare_hpos_compatible() {
		if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
			\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
		}
	}
	
	// CRON JOB - remove arbitrary products
	public function maybe_schedule_cron_hook() { 
		if ( !wp_next_scheduled ( 'remove_arbitrary_products_hook' ) ) {
			wp_schedule_event( time(), 'hourly', 'remove_arbitrary_products_hook' );
		}
	}
	public function plugin_url() {
		return untrailingslashit( plugins_url( '/', __FILE__ ) );
	}
	
	public function plugin_path() {
		return untrailingslashit( plugin_dir_path( __FILE__ ) );
	}
	
	// This is a bit hacky, it tries to set the customer as the active global user so that payment gateways 
	// can hopefully obtained saved cards for the customer properly 
	function force_customer_user( $gateways = array() ) { 
		$this->switch_user_context_to_customer( 'customer' );
		return $gateways;
	}
	
	function load_plugin_textdomain() {

		$locale = apply_filters( 'plugin_locale', get_locale(), 'manual_phone_orders' );

		load_textdomain( 'manual_phone_orders', WP_LANG_DIR.'/woocommerce/manual_phone_orders-'.$locale.'.mo' );

		$plugin_rel_path = apply_filters( 'ignitewoo_translation_file_rel_path', dirname( plugin_basename( __FILE__ ) ) . '/languages' );

		load_plugin_textdomain( 'manual_phone_order', false, $plugin_rel_path );
	}

	public function init() {
		global $user_ID, $ign_opc_core;

		//if ( version_compare( WOOCOMMERCE_VERSION, '2.4.12', '<' ) ) { 
		//	add_action( 'admin_notices', array( $this, 'version_mismatch' ), 1, 1 );
		//	return;
		//}

		// Ensure that this class loads so that arbitrary products can be deleted via WP Cron 
		// and when the order status reaches processing or completed.
		require_once( dirname( __FILE__ ) . '/includes/class-ign-arbitrary-products.php' );

		require_once( dirname( __FILE__ ) . '/includes/class-ign-opc-api.php' );
				
		$this->switch_user_context_to_customer( 'opc_user' );
		
		$this->opc_user_id = $user_ID;

		if ( !$this->has_access() && !current_user_can( 'administrator' ) ) {
			return;
		}

		add_filter('woocommerce_checkout_update_order_review', array( &$this, 'clear_wc_shipping_rates_cache' ) );
		
		// This causes problems with Angelleye's PayPal plugin, for some reason this causes the cart to be emptied
		// Ensure payment gateways are loaded and initialized.
		// It is required to do though for other plugin's such as WC Stripe Gateway
		if ( is_admin() && ( empty( $_GET['page'] ) || 'wc-settings' !==  $_GET['page'] ) ) { 
			if ( !class_exists( 'AngellEYE_Gateway_Paypal' ) && empty( WC()->payment_gateways->payment_gateways ) ) { 
				WC()->payment_gateways->init();
			}
		}
		
		/* Some gateways need to be loaded early to ensure they hook into wp_enqueue_scripts early 
		so that their related JS code loads into the order page
		*/
		//if ( class_exists( 'WC_Gateway_Stripe' ) /*&& empty( WC()->payment_gateways->payment_gateways )*/ && ( empty( $_GET['page'] ) || 'wc-settings' !==  $_GET['page'] ) ) { 
		if ( class_exists( 'WC_Gateway_Stripe' ) && ( !empty( $_GET['page'] ) && 'wc-settings' !==  $_GET['page'] ) ) { 
			WC()->payment_gateways->init();
		}

		require_once( dirname( __FILE__ ) . '/includes/class-ign-opc-core.php' );
		require_once( dirname( __FILE__ ) . '/includes/class-ign-cart.php' );
		require_once( dirname( __FILE__ ) . '/includes/class-ign-checkout.php' );

		// Helps with products that have no regular price but do have a "wholesale" price:
		if ( !empty( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ) { 
			$ign_opc_core->opc = true; 
		} else if ( !empty( $_REQUEST['post_data'] ) ) { 
			$args = wp_parse_args( $_REQUEST['post_data'] );
			if ( !empty( $args['ignitewoo_one_page_checkout_customer_id'] ) ) { 
				$ign_opc_core->opc = true; 
			}
		}
		
	}

	// Helper that clears the shipping calculations cache so that the custom shipping cost modules always returns the rate set by the user
	function clear_wc_shipping_rates_cache(){
		
		$packages = WC()->cart->get_shipping_packages();
		
		if ( empty( $packages ) ) { 
			return;
		}
		
		foreach ($packages as $key => $value) {
			$shipping_session = "shipping_for_package_$key";
	
			unset( WC()->session->$shipping_session );
		}
	}
	
	public function access_is_denied( $attrs = array() ) {

		if ( !$this->has_access() && !current_user_can( 'administrator' ) ) {
			return '<p class="ignitewoo_one_page_access_denied" style="font-weight:bold; font-style:italic">' . __( 'You do not have access to this page', 'manual_phone_orders' ) . '</p>';
		}
	}

	public function plugins_loaded() {

		if ( is_admin() && ( current_user_can( 'shop_manager' ) || current_user_can( 'administrator' ) ) ) {
			require_once( dirname( __FILE__ ) . '/includes/class-manual-order-integration.php' );
		}
	}

	public function version_mismatch() {
		?>
		<div class="updated fade error" id="message"><p><strong style="color:#DD3D36"><?php _e( 'WooCommerce Phone & Manual Orders Pro requires WooCommerce 2.4.12 or newer to operate correctly! ', 'manual_phone_orders' )?></strong></p></div>
		<?php
	}

	public function has_access() {
		global $user_ID;

		// Helps the built-in gateways load during order review updates via Ajax
		if ( isset( $_POST['post_data'] ) ) {

			$args = wp_parse_args( $_POST['post_data'] );

			if ( isset( $args['opc_admin_user_id'] ) ) {
				if ( user_can( absint( $args['opc_admin_user_id'] ), 'administrator' ) || user_can( absint( $args['opc_admin_user_id'] ), 'shop_manager' ) )
					return true;
			}
		}

		if ( empty( $user_ID ) || 'undefined' == $user_ID ) {
			if ( !empty( $this->opc_user_id ) ) { 
				$this->switch_user_context_to_customer( 'opc_user' );
			}
			$user_ID = get_current_user_id();
		}

		/*
		if ( empty( $this->opc_user_id ) )
			$this->switch_user_context_to_customer( 'opc_user' );

		if ( !empty( $_POST['opc_customer_selected'] ) )
		*/
			$this->switch_user_context_to_customer( 'opc_user' );

		if ( current_user_can( 'administrator' ) || current_user_can( 'shop_manager' ) )
			return true;

		$opts = get_option( 'woocommerce_ignitewoo_manual_order_settings' );

		if ( empty( $opts ) )
			return false;

		if ( 'yes' !== $opts['enable'] )
			return false;

		if ( !is_user_logged_in() )
			return false;

		$has_access = false;

		if ( !empty( $opts['roles'] ) )
		foreach( $opts['roles'] as $role ) {

			if ( current_user_can( $role ) )
				$has_access = true;
		}

		if ( !empty( $opts['users'] ) )
		foreach( $opts['users'] as $user_id ) {

			if ( $user_ID == $user_id  )
				$has_access = true;
		}

		return $has_access;
	}

	public function switch_user_context_to_customer( $who ) {
		global $user_ID, $current_user, $ign_opc_core;

		if ( is_admin() && !defined( 'DOING_AJAX' ) )
			return;
			
		// failsafe to ensure we get a customer ID if it exists in the request

		if ( isset( $_POST['post_data'] ) ) {
			$args = wp_parse_args( $_POST['post_data'] );
			if ( isset( $args['ignitewoo_one_page_checkout_customer_id'] ) ) {
				$_REQUEST['ignitewoo_one_page_checkout_customer_id'] = $args['ignitewoo_one_page_checkout_customer_id'];
			}
		}

		$this->opc_customer_id = isset( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ? absint( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) : null;

		if ( empty( $this->opc_customer_id ) && isset( $_REQUEST['one_page_customer_user'] ) )
			$this->opc_customer_id = $_REQUEST['one_page_customer_user'];
			
		if ( empty( $this->opc_customer_id ) && isset( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) )
			$this->opc_customer_id = $_REQUEST['ignitewoo_one_page_checkout_customer_id'];

		// Only set this once so that it remains consistent and we don't lose track of who the admin user is.
		if ( empty( $this->opc_user_id ) )
			$this->opc_user_id = $user_ID;

		if ( 'opc_user' == $who ) {
			$wp_user = wp_set_current_user( $this->opc_user_id );
			$user_ID = $this->opc_user_id;
			$current_user = $wp_user;
		} else if ( 'customer' == $who ) {
			$uid = isset( $this->opc_customer_id ) ? $this->opc_customer_id : $this->opc_user_id;
			// For whatever reason, when WooCommerce Payments is enable with PayPal standard turned on the user 
			// cannot be detected and this makes apply_coupon fail due to a nonce security issue
			if ( empty( $_GET['wc-ajax'] ) || $_GET['wc-ajax'] !== 'apply_coupon' ) {
				$wp_user = wp_set_current_user( $uid );
				$user_ID = $uid;
				$current_user = $wp_user;
			}
		}
	}

	function add_dummy_gateway( $methods ) {

		$methods = $this->invoice_gateway( $methods );
		$methods = $this->hold_order_gateway( $methods );
		$methods = $this->purchase_order_gateway( $methods );
		return $methods;
	}

	function hold_order_gateway( $methods = array() ) {
		global $typenow;

		// ------------- The "Place order on hold" gateway: ---------------

		// Gotta let it load when viewing orders so the method title appears correctly in the order
		if ( 'shop_order' == $typenow || ( is_admin() && ( isset( $_GET['page'] ) && 'wc-settings' == $_GET['page'] ) ) ) {
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-hold-order-gateway.php' );

			$methods['wc_gateway_opc_hold_order'] = 'WC_Gateway_OPC_Hold_Order';
			return $methods;
		}

		// For the admin area, so plugins such as role-based methods can pickup the gateway for adjusting permission settings
		if ( is_admin() && !defined( 'DOING_AJAX' ) ) {
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-hold-order-gateway.php' );

			$methods['wc_gateway_opc_hold_order'] = 'WC_Gateway_OPC_Hold_Order';
			return $methods;
		}
		
		if ( is_admin() && defined( 'DOING_AJAX' ) ) {
			$this->switch_user_context_to_customer( 'opc_user' );
			if ( !$this->has_access() ) { 
				$this->switch_user_context_to_customer( 'customer' );
				return $methods;
			}
			$this->switch_user_context_to_customer( 'customer' );
		}

		if ( isset( $_POST['opc_checkout_flag'] ) ) {
			// Do nothing
		} else if ( isset( $_POST['post_data'] ) ) {
			// Check for the checkout flag, if exists allow the custom gateway when enabled
			$args = wp_parse_args( $_POST['post_data'] );

			if ( empty( $args ) || empty( $args['opc_checkout_flag'] ) )
				return $methods;

		} else if ( isset( $_POST['payment_method'] ) && 'phone_manual_on_hold' == $_POST['payment_method'] ) {

			// do nothing, just a placeholder to allow the gateway to work since if that var is set the permission already passed.
			// this happens when the place order button is clicked since our custom checkout flag var isn't set at that point.

		} else {
			// No post data and no flag then don't allow the custom gateway
			return $methods;
		}

		require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-hold-order-gateway.php' );

		$methods['wc_gateway_opc_hold_order'] = 'WC_Gateway_OPC_Hold_Order';

		return $methods;

	}
	
	function purchase_order_gateway( $methods = array() ) {
		global $typenow;

		// ------------- The "Purchase order gateway: ---------------

		// Gotta let it load when viewing orders so the method title appears correctly in the order
		if ( 'shop_order' == $typenow || ( is_admin() && ( isset( $_GET['page'] ) && 'wc-settings' == $_GET['page'] ) ) ) {
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-purchase-order-gateway.php' );

			$methods['wc_gateway_opc_purchase_order'] = 'IGN_OPC_Gateway_Purchase_Order';
			return $methods;
		}

		// For the admin area, so plugins such as role-based methods can pickup the gateway for adjusting permission settings
		if ( is_admin() && !defined( 'DOING_AJAX' ) ) {
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-purchase-order-gateway.php' );

			$methods['wc_gateway_opc_purchase_order'] = 'IGN_OPC_Gateway_Purchase_Order';
			return $methods;
		}
		
		if ( is_admin() && defined( 'DOING_AJAX' ) ) {
			$this->switch_user_context_to_customer( 'opc_user' );
			if ( !$this->has_access() ) { 
				$this->switch_user_context_to_customer( 'customer' );
				return $methods;
			}
			$this->switch_user_context_to_customer( 'customer' );
		}

		if ( isset( $_POST['opc_checkout_flag'] ) ) {
			// Do nothing
		} else if ( isset( $_POST['post_data'] ) ) {
			// Check for the checkout flag, if exists allow the custom gateway when enabled
			$args = wp_parse_args( $_POST['post_data'] );

			if ( empty( $args ) || empty( $args['opc_checkout_flag'] ) )
				return $methods;

		} else if ( isset( $_POST['payment_method'] ) && 'phone_manual_purchase_order' == $_POST['payment_method'] ) {

			// do nothing, just a placeholder to allow the gateway to work since if that var is set the permission already passed.
			// this happens when the place order button is clicked since our custom checkout flag var isn't set at that point.

		} else {
			// No post data and no flag then don't allow the custom gateway
			return $methods;
		}

		require_once( dirname( __FILE__ ) . '/includes/gateways/class-purchase-order-gateway.php' );

		$methods['wc_gateway_opc_purchase_order'] = 'IGN_OPC_Gateway_Purchase_Order';

		return $methods;

	}
	
	function invoice_gateway( $methods = array() ) {
		global $typenow, $wp;

		// Gotta let it load when viewing orders so the method title appears correctly in the order
		if ( 'shop_order' == $typenow || ( is_admin() && ( isset( $_GET['page'] ) && 'wc-settings' == $_GET['page'] ) ) ) {
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-dummy-gateway.php' );
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-paid-in-person-gateway.php' );
			
			$methods['wc_gateway_opc_dummy'] = 'WC_Gateway_OPC_Dummy';
			$methods['wc_gateway_opc_paid_in_person'] = 'WC_Gateway_OPC_Paid_In_Person';
			return $methods;
		}

		// For the admin area, so plugins such as role-based methods can pickup the gateway for adjusting permission settings
		if ( is_admin() && !defined( 'DOING_AJAX' ) ) {
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-dummy-gateway.php' );
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-paid-in-person-gateway.php' );
			
			$methods['wc_gateway_opc_dummy'] = 'WC_Gateway_OPC_Dummy';
			$methods['wc_gateway_opc_paid_in_person'] = 'WC_Gateway_OPC_Paid_In_Person';
			return $methods;
		}
		
		if ( is_admin() && defined( 'DOING_AJAX' ) ) {
			$this->switch_user_context_to_customer( 'opc_user' );
			if ( !$this->has_access() ) { 
				$this->switch_user_context_to_customer( 'customer' );
				return $methods;
			}
			$this->switch_user_context_to_customer( 'customer' );
		}

		// Check for the checkout flag, if exists allow the custom gateway when enabled
		if ( isset( $_POST['opc_checkout_flag'] ) ) {
			// Do nothing
		} else if ( isset( $_POST['post_data'] ) ) {

			$args = wp_parse_args( $_POST['post_data'] );

			if ( empty( $args ) || empty( $args['opc_checkout_flag'] ) )
				return $methods;

		} else if ( isset( $_POST['payment_method'] ) && 'phone_manual_dummy' == $_POST['payment_method'] ) {

			// do nothing, just a placeholder to allow the gateway to work since if that var is set the permission already passed.
			// this happens when the place order button is clicked since our custom checkout flag var isn't set at that point.

		// If we're on the Thank You page for an order, might need to display info and/or send an invoice automatically
		// so let the gateways load.
		} else if ( isset( $wp->query_vars['order-received'] ) ) {
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-dummy-gateway.php' );
			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-paid-in-person-gateway.php' );

			$methods['wc_gateway_opc_dummy'] = 'WC_Gateway_OPC_Dummy';
			$methods['wc_gateway_opc_paid_in_person'] = 'WC_Gateway_OPC_Paid_In_Person';
		} else {
			// No post data and no flag then don't allow the custom gateway
			return $methods;
		}

		require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-dummy-gateway.php' );
		require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-paid-in-person-gateway.php' );

		$methods['wc_gateway_opc_dummy'] = 'WC_Gateway_OPC_Dummy';
		$methods['wc_gateway_opc_paid_in_person'] = 'WC_Gateway_OPC_Paid_In_Person';

		return $methods;
	}

	function add_shipping_method( $methods = array() ) {
		global $typenow;

		// Gotta let it load when viewing orders so the method title appears correctly in the order
		if ( 'shop_order' == $typenow || ( is_admin() && ( isset( $_GET['page'] ) && 'wc-settings' == $_GET['page'] ) ) ) {

			require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-custom-shipping-method.php' );

			$methods['phone_manual_dummy_shipping'] = 'WC_Shipping_OPC_Dummy_Gateway';

			return $methods;
		}

		$this->switch_user_context_to_customer( 'opc_user' );
		
		if ( is_admin() && defined( 'DOING_AJAX' ) ) {
			if ( !$this->has_access() )
				return $methods;
		}

		//$this->switch_user_context_to_customer( 'customer' );
		
		if ( isset( $_POST['opc_checkout_flag'] ) ) {

		} else if ( isset( $_POST['post_data'] ) ) {

			$args = wp_parse_args( $_POST['post_data'] );

			if ( empty( $args ) || empty( $args['opc_checkout_flag'] ) )
				return $methods;
		}

		require_once( dirname( __FILE__ ) . '/includes/gateways/class-opc-custom-shipping-method.php' );

		$methods['phone_manual_dummy_shipping'] = 'WC_Shipping_OPC_Dummy_Gateway';

		return $methods;
	}
	
	function checkout_billing() {
		global $user_ID, $ign_opc, $ign_opc_core;

		if ( empty( $ign_opc_core ) || empty( $ign_opc_core->opc ) )
			return;

		remove_action( 'woocommerce_checkout_billing' , array( WC()->checkout, 'checkout_form_billing' ) );
		
		// Compatibility with Astra theme, it does an odd thing injecting the shipping form into the billing
		// as seen in the Astra_Woocommerce class, woocommerce_checkout method
		remove_action( 'woocommerce_checkout_billing' , array( WC()->checkout, 'checkout_form_shipping' ) );

		// Check theme
		$template = locate_template( array( 'phone-orders/form-billing.php' ), false, false );

		if ( '' != $template ) {
			require( $template );
		} else if ( file_exists( dirname( __FILE__ ) . '/phone-orders/form-billing.php' ) ) {
			$checkout = WC()->checkout;
			require( dirname( __FILE__ ) . '/phone-orders/form-billing.php' );
		} else { // fallback to stock
			wc_get_template( 'checkout/form-billing.php', array( 'checkout' =>  WC()->checkout ) );
		}

		echo '<input type="hidden" name="opc_admin_user_id" value="' . $ign_opc->opc_user_id . '">';
	}

	function checkout_shipping() {
		global $ign_opc_core;

		if ( empty( $ign_opc_core ) || empty( $ign_opc_core->opc ) )
			return;
		
		remove_action( 'woocommerce_checkout_shipping' , array( WC()->checkout, 'checkout_form_shipping' ) );
		

		// Check theme
		$template = locate_template( array( 'phone-orders/form-shipping.php' ), false, false );

		if ( '' != $template ) {
			require( $template );
		} else if ( file_exists( dirname( __FILE__ ) . '/phone-orders/form-shipping.php' ) ) {
			$checkout = WC()->checkout;
			require( dirname( __FILE__ ) . '/phone-orders/form-shipping.php' );
		} else { // fallback to stock
			wc_get_template( 'checkout/form-shipping.php', array( 'checkout' =>  WC()->checkout ) );
		}
	}

	/* Not implemented yet
	// Always allow backorders because it's an admin or shop manager adding the order
	public function backorders_allowed( $allowed, $product ) {

		//if ( !isset( $_POST['action'] ) || empty( $_POST['opc_checkout_flag'] ) )
		//	return $allowed;

		//if ( !isset( $_POST['action'] ) || ( 'woocommerce_add_to_cart_opc' != $_POST['action'] && 'woocommerce_add_to_cart_opc_variable' !== $_POST['action'] ) && 'ign_one_page_searchable_add' !== $_POST['action'] )
		//	return $allowed;

		return true;
	}

	// Always allow sale or more than one because it's an admin or shop manager adding the order
	public function sold_individually( $sold_individually, $product ) {

		//if ( !isset( $_POST['action'] ) || empty( $_POST['opc_checkout_flag'] ) )
		//	return $sold_individually;

		//if ( !isset( $_POST['action'] ) || ( 'woocommerce_add_to_cart_opc' != $_POST['action'] && 'woocommerce_add_to_cart_opc_variable' !== $_POST['action'] ) && 'ign_one_page_searchable_add' !== $_POST['action'] )
		//	return $sold_individually;

		return false;
	}
	*/
	
	
	// This allows products with no retail or sale price set to be purchased via the manual/phone order page,
	// but at the moment this doesn't work 100% consistent, particulary if the page is reloaded.
	public function load_request_vars() {  

		if ( isset( $_POST['post_data'] ) ) {
			$args = wp_parse_args( $_POST['post_data'] );
			if ( isset( $args['ignitewoo_one_page_checkout_customer_id'] ) )
				$_REQUEST['ignitewoo_one_page_checkout_customer_id'] = $args['ignitewoo_one_page_checkout_customer_id'];
		}

		if ( !empty( $_REQUEST['one_page_customer_user'] ) ) { 
			$_REQUEST['ignitewoo_one_page_checkout_customer_id'] = $_REQUEST['one_page_customer_user'];
		}
		
		if ( !empty( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ) {
			remove_all_actions( 'wp_ajax_woocommerce_get_refreshed_fragments' );
		}
	}
	
	public function is_purchasable( $is_purchasable, $product ) { 
		global $ign_opc_core;

		$this->load_request_vars();
	
		if ( !empty( $_REQUEST['ignitewoo_one_page_checkout_customer_id'] ) ) { 
			$ign_opc_core->opc = true; 
			return true;
		} else {
			return $is_purchasable;
		}
	}
	
}


global $ign_opc;
$ign_opc = new IGN_Manual_Phone_Orders();

// END









































































































if ( ! function_exists( 'ignitewoo_queue_update' ) )
	require_once( dirname( __FILE__ ) . '/ignitewoo_updater/ignitewoo_update_api.php' );

$this_plugin_base = plugin_basename( __FILE__ );

add_action( "after_plugin_row_" . $this_plugin_base, 'ignite_plugin_update_row', 1, 2 );

ignitewoo_queue_update( plugin_basename( __FILE__ ), '2a0aca041ccc02ea8b50ffcac00b9583', '22611' );
